/*
 * File:        exiftool.c
 *
 * Description: Simple stub to launch the pure Perl version of exiftool
 *
 * Usage:       exiftool [-h|normal exiftool arguments]
 *
 * Revisions:   2011/09/15 - P. Harvey Created
 *              2011/09/16 - PH Fixed mingw problem and use environment variables
 *              2011/09/18 - PH Parse arguments in file name and change default EXIFTOOL
 *		2011/09/18 - XX Added option for registry debugger interception
 */
#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#ifdef __MINGW32__
#include <process.h>
#endif
/*
 * define the default Perl executable and exiftool application script pathnames
 * (these may be changed at runtime by setting the corresponding environment variables)
 */
#define PERLNAME    "perl"
#define EXIFTOOL    "c:\\perl\\site\\bin\\exiftool"

int main(int argc, char **argv)
{
    int rtnVal = 0;

    /* generate our argument list */
    char **args = (char **)malloc((argc+2) * sizeof(char**));
    if (!args) { fprintf(stderr, "Out of memory\n"), exit(1); }

    /* argument 0: perl executable name */
    args[0] = getenv("PERLNAME");
    if (!args[0]) args[0] = PERLNAME;

    /* argument 1: exiftool script name */
    args[1] = getenv("EXIFTOOL");
    if (!args[1]) args[1] = EXIFTOOL;

    if (argc == 2 && !strcmp(argv[1],"-h")) {
        printf("Name:   exiftool.exe - Run the Perl version of exiftool\n\n");
        printf("Environment variables:\n");
        printf("  PERLNAME - Perl executable (\"%s\")\n", args[0]);
        printf("  EXIFTOOL - exiftool script (\"%s\")\n", args[1]);
        free(args);
        exit(0);
    }

    /* add arguments embedded in filename */
    /* (to replicate this feature of exiftool.exe) */
    int argc2 = 2;
    int n = strlen(argv[0]);
    char *ket = strrchr(argv[0], ')');
    char *bra;
    if (ket && (!strcasecmp(ket+1,".pl") || !strcasecmp(ket+1,".exe") || ket[1]=='\0') &&
        (bra = strrchr(argv[0], '(')) != NULL && bra < ket)
    {
        int n = ket - bra - 1;
        char *buf = (char *)malloc(n + 1);
        memcpy(buf, bra+1, n);
        buf[n] = '\0';
        char *pt;
        for (pt=strtok(buf, " "); pt; pt=strtok(NULL, " ")) {
            args = (char **)realloc(args, (argc+argc2+1) * sizeof(char**));
            if (!args) { fprintf(stderr, "Out of memory\n"), exit(1); }
            args[argc2++] = pt;
        }
    }

    /* scrapt first two args if intercepted by registry debugger */
    int i;
    if (argc > 2 && !strcmp(argv[1],"-skip2args")) {
        argc = argc-2;
        for (i=1; i<argc; i++) {
            argv[i] = argv[i+2];
        }
    }

    /* copy over the exiftool arguments */
    for (i=1; i<argc; ++i) {
        args[argc2++] = argv[i];
    }
    args[argc2] = NULL;     /* NULL to end argument list */

#ifdef __MINGW32__
    /* must use _spawnvp() instead of execvp() with mingw because with
     * mingw execvp() terminates the current process and forks a new one */
    rtnVal = _spawnvp(_P_WAIT, args[0], (const char* const*)args);
#else
    execvp(args[0], args);  /* should never return */
    rtnVal = 1;
#endif

    if (rtnVal) fprintf(stderr, "Error executing %s\n", args[0]);

    free(args);

    return(rtnVal);
}
